function ScriptChannel(id) {
    this._constructor(id);
}
(function() {

    var DEFAULT_ID = "javascriptchannel";
    var MAX_SCRIPTS = 10;

    ScriptChannel.prototype = {

        id: null,
        url: null,
        scripts: null,
        currentIndex: null,
        preventCaching: null,

        _constructor: function(id) {
            this.id = id || DEFAULT_ID;
            this.preventCaching = false;
        },

        setPreventCaching: function(value) {
            this.preventCaching = !!value;
        },

        open: function(url) {
            this.url = url;
            this.scripts = new Array();
            this.currentIndex = 0;
        },

        send: function(data) {
            var holder = this._next();

            (function(script, url){
                script.setAttribute("language", "javascript");
                script.setAttribute("type", "text/javascript");
                script.setAttribute("src", encodeURI(url));
            }.delay(10, holder.getElementsByTagName("script")[0], this._createUrl(data)));
        },

        close: function() {
            if (this.scripts) {
                while (this.scripts.length > 0) {
                    this._remove(0);
                    this.scripts.shift();
                }
            }
        },

        _createUrl: function(data) {
            var result = this.url + (data || "");
            if (this.preventCaching) {
                result += (result.indexOf("?") == -1 ? "?" : "&");
                result += "nocache=" + new Date().getTime();
            }
            return encodeURI(result);
        },

        _next: function() {
            if (++this.currentIndex >= MAX_SCRIPTS) {
                this.currentIndex = 0;
            }

            this._remove(this.currentIndex);

            var result = this._create(this.id + this.currentIndex);
            this.scripts[this.currentIndex] = result;

            return result;
        },

        _remove: function(index) {
            if (this.scripts[index] == null) {
                return;
            }

            var id = this.scripts[index].getAttribute("id");
            this.scripts[index] = null;

            // crash in IE5 without delay
            (function(holder) {
                if (holder != null) {
                    holder.parentNode.removeChild(holder);
                }
            }.delay(50, $(id)));
        },

        _create: function(id) {
            var result = document.body.appendChild(document.createElement(Tag.SPAN));
            result.setAttribute("id", id);
            result.style.display = "none";
            result.innerHTML = "Text for IE.<s" + "cript></" + "script>";

            return result;
        }
    };
})();