function NewDirectoryDialog() {
    this._constructor();
}(function() { // create local scope;

    NewDirectoryDialog.Labels = {NAME: 'New directory'};
    NewDirectoryDialog.Buttons = {OK: 'OK', CANCEL: 'Cancel'};

    NewDirectoryDialog.prototype = {

        dialog: null,
        dirField: null,

        _constructor: function() {
            this.dialog = new ModalDialog("NewDirectoryDialog");
            this.dirField = Utils.createTextInput("newDirInputField");
            Event.observe(this.dirField, "keyup", this._updateButtons.bind(this));

            this.dialog.getContent().appendChild(Utils.createLabel(NewDirectoryDialog.Labels.NAME, this.dirField.name));
            this.dialog.getContent().appendChild(this.dirField);
            this.dialog.addButton(ModalDialog.BUTTON_OK, NewDirectoryDialog.Buttons.OK, this.onSubmit.bind(this));
            this.dialog.addButton(ModalDialog.BUTTON_CANCEL, NewDirectoryDialog.Buttons.CANCEL, this.onCancel.bind(this));
            this.dialog.setButtonEnabled(ModalDialog.BUTTON_OK, false);
        },

        isVisible: function() {
            return this.dialog.isVisible();
        },

        setVisible: function(visible) {
            this.dialog.setVisible(visible);

            if (visible) {
                this._requestFocus();
                this._updateButtons();
            } else {
                this.dirField.onkeyup = null;
            }

            this._registerKeyHandlers();
        },

        getDirectoryName: function() {
            var result = $F(this.dirField);
            return (result != null) ? result : "";
        },

        onSubmit: function() {
            this.setVisible(false);
            var name = this.getDirectoryName();

            if (this._isValidDirName(name)) {
                new Action(Action.NEW_DIRECTORY, getActionForm()).perform(createParameters(name));
            }
        },

        onCancel: function() {
            this.dirField.value = '';
            this.setVisible(false);
        },

        processKeyEvent: function(ke) {
            var code = ke.getKeyCode();

            switch (code) {
                case KeyEvent.VK_ENTER:
                    ke.cancel();
                    this.onSubmit();
                    break;

                case KeyEvent.VK_ESCAPE:
                    ke.cancel();
                    this.onCancel();
                    break;
            }
        },

        _isValidDirName: function(name) {
            return name != null && name.trim().length > 0;
        },

        _updateButtons: function() {
            var enabled = this._isValidDirName(this.getDirectoryName());
            this.dialog.setButtonEnabled(ModalDialog.BUTTON_OK, enabled);
            this._requestFocus();
        },

        _registerKeyHandlers: function() {
            var km = KeyboardManager.getInstance();
            if (this.isVisible()) {
                km.registerHandler(KeyEvent.VK_ENTER, this);
                km.registerHandler(KeyEvent.VK_ESCAPE, this);
            } else {
                km.unregisterHandler(KeyEvent.VK_ENTER, this);
                km.unregisterHandler(KeyEvent.VK_ESCAPE, this);
            }
        },

        _requestFocus: function() {
            if (this.dialog.isVisible()) {
                try {
                    this.dirField.focus();
                } catch(e) {}
            }
        }

    };
})();